<?php

namespace SenSai;

use FML\ManiaLink;
use FML\Controls\Frame;
use FML\Controls\Quad;
use FML\Controls\Labels\Label_Text;
use FML\Controls\Control;
use ManiaControl\ManiaControl;
use ManiaControl\Callbacks\Callbacks;
use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Callbacks\CallbackManager;
use ManiaControl\Callbacks\TimerListener;
use ManiaControl\Callbacks\TimerManager;
use ManiaControl\Plugins\Plugin;
use ManiaControl\Maps\MapManager;
use ManiaControl\Players\Player;
use ManiaControl\Players\PlayerManager;
use ManiaControl\Settings\Setting;
use ManiaControl\Settings\SettingManager;
use ManiaControl\Commands\CommandListener;

/**
 * Loadscreens Plugin
 *
 * @author Sen-Sai
 */

class Loadscreens implements CallbackListener, TimerListener, CommandListener, Plugin {
    /**
     * Constants
     */
    const PLUGIN_ID     = 45;
    const PLUGIN_VERSION = 1.55;
    const PLUGIN_NAME = 'Loadscreens';
    const PLUGIN_AUTHOR = 'G]R [SG-1]Sen-sai';

    // Loadscreen properties

    const MLID_LOADSCREENS     = 'Loadscreens.ID';
    const SETTING_LS_ACTIVATED = 'Loadscreens activated';
	const SETTING_USE_SOUNDS   = 'Use Glimratten Soundbox';
    const SETTING_LS_POSX      = 'Loadscreens Position: X';
    const SETTING_LS_POSY      = 'Loadscreens Position: Y';
    const SETTING_LS_WIDTH     = 'Loadscreens Size: Width';
    const SETTING_LS_HEIGHT    = 'Loadscreens Size: Height';
    const SETTING_LS_URL       = 'Loadscreens URL';
    const SETTING_LS_OFFSET    = 'Loadscreens EndMatch Delay';
	const SETTING_LS_TYPE_B    = 'Loadscreens Start on';
	const SETTING_LS_TYPE_E    = 'Loadscreens Stop on';

	private $screen = array();
	private $sound = array();
	private $cnt = 0;

    /**
     * Private Properties
     */
    /**
     * @var maniaControl $maniaControl
     */
    private $maniaControl = null;
	private $mapbegon = true;
	private $loadscreenOn = false;
	private $displaying = false;
	private $display = 0;
	private $useSounds = false;

    /**
     * Prepares the Plugin
     *
     * @param ManiaControl $maniaControl
     * @return mixed
     */
    public static function prepare(ManiaControl $maniaControl) {
        //do nothing
    }

    /**
     * Load the plugin
     *
     * @param ManiaControl $maniaControl
     * @return bool
     */
    public function load(ManiaControl $maniaControl) {
        $this->maniaControl = $maniaControl;

        //Register callbacks

        $this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_LS_ACTIVATED, true);
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_USE_SOUNDS, false);
        $this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_LS_URL, '');
	    $this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_LS_TYPE_B, array('begin of podium', 'end of match', 'begin unloading map'));
        $this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_LS_OFFSET, 12);
	    $this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_LS_TYPE_E, array('begin warmup', 'after loading new map'));

		$file = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_LS_URL);
		$this->useSounds = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_USE_SOUNDS);
		$array = get_headers($file);
		$string = $array[0];
		if(!strpos($string,"200")) {

			$this->maniaControl->getChat()->sendInformation('The URL does not exist. Please check. Loadscreens disabled.', null, false);
			return;
		}



		$start_on = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_LS_TYPE_B);
	    $end_on = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_LS_TYPE_E);
		if ($start_on=="") $start_on='begin of podium';
	    if ($end_on=="") $end_on='begin warmup';

	    // When to stop showing loadscreen
	    if ($end_on === 'begin warmup') {
		   // $this->maniaControl->getChat()->sendInformation('DEBUG :: Endloadingscreen on warmup', null, false);
		    $this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_WARMUP_START, $this, 'cbStartStopScreen');
	    } else {
		   // $this->maniaControl->getChat()->sendInformation('DEBUG :: Endloadingscreen on loadmapend', null, false);
		    $this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_LOADINGMAPEND, $this, 'cbStartStopScreen');
	    }

	    // When to start showing loadscreen
	    if ($start_on === 'begin unloading map') {
		  //  $this->maniaControl->getChat()->sendInformation('DEBUG :: loadscreen on unloading map', null, false);
		    $this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_UNLOADINGMAPSTART, $this, 'cbStartLoadScreen');
	    }
	    if ($start_on === 'end of match') {
		   // $this->maniaControl->getChat()->sendInformation('DEBUG :: loadscreen on endmatch', null, false);
		    $this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_ENDMATCHEND, $this, 'cbStartLoadScreen');
	    }
		if ($start_on === 'begin of podium') {
			//$this->maniaControl->getChat()->sendInformation('DEBUG :: loadscreen on beginpodium', null, false);
			$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_PODIUMSTART, $this, 'cbStartLoadScreen');
		}



	    //$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ENDMAP, $this, 'cbEndMap');

	    //$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_PODIUMEND, $this, 'cbPodiumEnd');

	    //$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_UNLOADINGMAPEND, $this, 'cbUnloadingMapEnd');
	    //$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_LOADINGMAPSTART, $this, 'cbLoadingMapStart');
	    //$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_LOADINGMAPEND, $this, 'cbLoadingMapEnd');
	    //$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_ENDMATCHSTART, $this, 'cbEndMatchStart');



	    //$this->maniaControl->callbackManager->registerCallbackListener(CallbackManager::CB_MP_MODESCRIPTCALLBACK, $this, 'handleCallbacks');
		//$this->maniaControl->callbackManager->registerCallbackListener(CallbackManager::CB_MP_MODESCRIPTCALLBACKARRAY, $this, 'handleCallbacks');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(SettingManager::CB_SETTING_CHANGED, $this, 'handleSettingChangedCallback');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(PlayerManager::CB_PLAYERCONNECT, $this, 'preloadtoLogin');
		$this->maniaControl->getCommandManager()->registerCommandListener('hidescreen', $this, 'hide', true);
		$this->maniaControl->getCommandManager()->registerCommandListener('showscreen', $this, 'displayLoadscreens', true);

		$ext = substr(strrchr($file, "."), 1);
		if ($ext=="xml")
		{
			if ($xml = simplexml_load_string(file_get_contents($file)))
			{
				foreach($xml->loadscreen_image as $screen)
				{
					$this->screen[] = $screen;
					//echo $screen;
				}
				if($this->useSounds)
				{
					foreach($xml->loadscreen_sound as $sound)
				{
					$this->sound[] = $sound;
					//echo $sound;
				}

				}
			}
			$this->cnt = count($this->screen);
			if($this->useSounds)
			{
				$cnt=count($this->sound);
				$this->maniaControl->getChat()->sendInformation('Got '.$this->cnt.' screens and '.$cnt.' sounds..', null, false);
			}
			else
			{
				$this->maniaControl->getChat()->sendInformation('Got '.$this->cnt.' screens..', null, false);
			}
		}
		elseif ($ext=="jpg" || $ext=="png")
		{
			$this->cnt=1;
			$this->screen = $file;
		}
		else
		{
			$this->maniaControl->getChat()->sendInformation('The URL must be either .xml, .png or .jpg', null, false);
			$this->maniaControl->getCallbackManager()->unregisterCallbackListener($this);
			return;
		}
		$this->preload();
		//$this->maniaControl->getTimerManager()->registerTimerListening($this, 'handle1Second', 1000);
    }
	public function preloadtoLogin(player $player) {
        $pos_x = 128;
        $pos_y = 128;
		$pos_z = 1000;
        $width = 128;
        $height = 180;
		$manialink = new ManiaLink(self::MLID_LOADSCREENS);
		$frame = new Frame();
        $manialink->addChild($frame);
        $frame->setPosition($pos_x, $pos_y);
		$frame->setSize($width, $height);
		$frame->setZ($pos_z);

		if ($this->cnt > 1)
		{
			foreach ($this->screen as $image)
			{
				$quad = new Quad();
        		$frame->addChild($quad);
        		$quad->setSize($width, $height);
        		$quad->setImage($image);
				$quad->setVisible(false);
			}
		}
		else
		{
			$image= $this->screen;
			$quad = new Quad();
			$frame->addChild($quad);
			$quad->setSize($width, $height);
			$quad->setImage($image);
			$quad->setVisible(false);
		}
		$this->maniaControl->getManialinkManager()->sendManialink($manialink,$player);
	}

	public function preload() {
		//$login=$player->login;
        $pos_x = 128;
        $pos_y = 128;
		$pos_z = 1000;
        $width = 128;
        $height = 180;
		$manialink = new ManiaLink(self::MLID_LOADSCREENS);
		$frame = new Frame();
        $manialink->addChild($frame);
        $frame->setPosition($pos_x, $pos_y);
		$frame->setSize($width, $height);
		$frame->setZ($pos_z);
		if ($this->cnt > 1)
		{
			foreach ($this->screen as $image)
			{
				$quad = new Quad();
        		$frame->addChild($quad);
        		$quad->setSize($width, $height);
        		$quad->setImage($image);
				$quad->setZ($pos_z);
				$quad->setVisible(false);
			}
		}
		else
		{
			$image= $this->screen;
			$quad = new Quad();
			$frame->addChild($quad);
			$quad->setSize($width, $height);
			$quad->setImage($image);
			$quad->setZ($pos_z);
			$quad->setVisible(false);
		}
		$this->maniaControl->getChat()->sendInformation('Preloading screens to all.', null, false);
		$this->maniaControl->getManialinkManager()->sendManialink($manialink);
	}

    public function displayLoadscreens(){
		if(!$this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_LS_ACTIVATED))
		{ return; }


		$queuedMap = $this->maniaControl->getMapManager()
		                                ->getMapQueue()
		                                ->getNextMap();

		$requester = null;
		// if the nextmap is not a queued map, get it from map info
		if (!$queuedMap) {
			$map    = $this->maniaControl->getClient()
			                             ->getNextMapInfo();
			$name   = $map->name;
			$author = $map->author;
		} else {
			$requester = $queuedMap[0];
			$map       = $queuedMap[1];
			$name      = $map->name;
			$author    = $map->authorLogin;
		}

        $pos_x = 0;
        $pos_y = 0;
        $width = 320;
        $height = 180;
		if($this->cnt > 1)
		{
			$nr=mt_rand(1,($this->cnt-1));
			$image  = $this->screen[$nr];
			if($this->maniaControl->getPluginManager()->getPlugin('SenSai\GRSoundsPlugin'))
			{
				$sounds = $this->maniaControl->getPluginManager()->getPlugin('SenSai\GRSoundsPlugin');
				if ($this->useSounds)
				{
					$play=trim($this->sound[$nr]);
					//echo $play;
					if($play!="none")
					{
						$sounds->playSound($play,1,true);
					}
					else
					{
						$sounds->playSound("LoadScreenVisible1",1,true);
					}
				}
				else
				{
					$sounds->playSound("LoadScreenVisible1",1,true);
				}
			}
		}
		else
		{
        	$image  = $this->screen;
		}




		$ext = substr(strrchr($image, "."), 1);
        $manialink = new ManiaLink(self::MLID_LOADSCREENS);

        $frame = new Frame();
        $manialink->addChild($frame);
		//$manialink->setTimeout(10000);
        $frame->setSize($width, $height);
        $frame->setPosition($pos_x, $pos_y);
	    $frame->setZ(1000.0);

		$quad = new Quad();
		$frame->addChild($quad);
		$quad->setSize($width, $height);
		$quad->setImage($image);
		$quad->setZ(1000.0);

		$lbl_loading = new Label_Text();
		$frame->addChild($lbl_loading);
		$lbl_loading->setPosition(120,85);
		$lbl_loading->setZ(1030.0);
		$lbl_loading->setText("Now loading..");
		$lbl_loading->setTextSize(0.3);

		$lbl_mapname = new Label_Text();
		$frame->addChild($lbl_mapname);
		$lbl_mapname->setPosition(120,80);
		$lbl_mapname->setZ(1030.0);
		$lbl_mapname->setText($name);
		$lbl_mapname->setTextSize(1.0);

		$lbl_mapauthor = new Label_Text();
		$frame->addChild($lbl_mapauthor);
		$lbl_mapauthor->setPosition(120,75);
		$lbl_mapauthor->setZ(1030.0);
		$lbl_mapauthor->setText("by ".$author);
		$lbl_mapauthor->setTextSize(0.5);

		$label = new Label_Text();
		$frame->addChild($label);
		$label->setPosition(120,70);
		$label->setZ(1030.0);
		$label->setText("Loadscreens");
		$label->setTextSize(1.0);
		$labeel = new Label_Text();
		$frame->addChild($labeel);
		$labeel->setPosition(120,65);
		$labeel->setText("by G]R [SG-1]Sen-Sai");
		$labeel->setTextSize(0.3);
		$labeel->setZ(1030.0);


        $this->maniaControl->getManialinkManager()->sendManialink($manialink);
		$this->displaying = true;
    }

    public function hideScreen() {
		if($this->loadscreenOn)
		{
        	$emptyManialink = new ManiaLink(self::MLID_LOADSCREENS);
			$emptyManialink->removeChildren();
        	$this->maniaControl->getManialinkManager()->sendManialink($emptyManialink);
			$this->loadscreenOn = false;
			$this->displaying = false;
		}
    }

    public function hide() {
        	$emptyManialink = new ManiaLink(self::MLID_LOADSCREENS);
			$emptyManialink->removeChildren();
        	$this->maniaControl->getManialinkManager()->sendManialink($emptyManialink);
			$this->loadscreenOn = false;
    }




	public function handleWait() {
		if ($this->loadscreenOn){ return; }
		if ($this->displaying) { return; }
		//$this->maniaControl->getChat()->sendInformation('DEBUG :: Startscreen', null, false);
		$this->displayLoadscreens();
		$this->loadscreenOn=true;
		$this->displaying = true;
	}
/*
	public function handle1Second() {
		if (!$this->loadscreenOn){ return; }
		if ($this->displaying) {return;}
		if( (int)time() >= $this->display )
		{
			$this->displayLoadscreens();

		}
	}
*/
/*
$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_PODIUMSTART, $this, 'cbPodiumStart');
$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_PODIUMEND, $this, 'cbPodiumEnd');
$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_UNLOADINGMAPSTART, $this, 'cbUnloadingMapStart');
$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_UNLOADINGMAPEND, $this, 'cbUnloadingMapEnd');
$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_LOADINGMAPSTART, $this, 'cbLoadingMapStart');
$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_LOADINGMAPEND, $this, 'cbLoadingMapEnd');
$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_ENDMATCHSTART, $this, 'cbEndMatchStart');
$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_ENDMATCHEND, $this, 'cbEndMatchEnd');
*/

	public function cbStartLoadScreen() {
		$waitfor = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_LS_OFFSET)*1000;
		//$this->maniaControl->getChat()->sendInformation('DEBUG :: Startscreen wait for : '.$waitfor, null, false);
		$this->maniaControl->getTimerManager()->registerOneTimeListening($this, 'handleWait', $waitfor);

	}

	public function cbStartStopScreen() {
		//$this->maniaControl->getChat()->sendInformation('DEBUG :: Endscreen', null, false);
		$this->hideScreen();

	}


    public function unload() {
        $this->hideScreen();
        $this->maniaControl->getCallbackManager()->unregisterCallbackListener($this);
        unset($this->maniaControl);
    }

	// handle scriptsettings changes
	public function handleSettingChangedCallback(Setting $setting)
	{
		if ($setting->belongsToClass($this))
		{
			$this->screen = array();
			$this->sound = array();
			$this->load($this->maniaControl);
			return;
		}

	}


     /**
     * Get plugin id
     *
     * @return int
     */
    public static function getId() {
        return self::PLUGIN_ID;
    }

    /**
     * Get Plugin Name
     *
     * @return string
     */
    public static function getName() {
        return self::PLUGIN_NAME;
    }

    /**
     * Get Plugin Version
     *
     * @return float,,
     */
    public static function getVersion() {
        return self::PLUGIN_VERSION;
    }

    /**
     * Get Plugin Author
     *
     * @return string
     */
    public static function getAuthor() {
        return self::PLUGIN_AUTHOR;
    }

    /**
     * Get Plugin Description
     *
     * @return string
     */
    public static function getDescription() {
        return 'Displays a custom images during map changes.';
    }
}
